# -*- coding: utf-8 -*-
"""
    pyRofex.components.enums

    Defines all library enumerations
"""
from enum import Enum


class Environment(Enum):
    """Available environments to used.
    REMARKET: Demo environment used for testing purpose.
    LIVE: ROFEX production environment.
    """
    REMARKET = 1
    LIVE = 2


class TimeInForce(Enum):
    """Time Modifier of the order that defines the time the order will be active.
    DAY: Order valid during the day. It will expires when the market close.
    """
    DAY = 'Day'


class Market(Enum):
    """Market ID associated to the instruments.
    ROFEX: ROFEX Exchange.
    """
    ROFEX = 'ROFX'


class Side(Enum):
    """Identifies the side of an order.
    BUY: Buy Order
    SELL: Sell Order
    """
    BUY = "buy"
    SELL = "sell"


class OrderType(Enum):
    """Identifies the different order types.
    LIMIT: For Limit Orders.
    MARKET: For Market Orders.
    MARKET_TO_LIMIT: For Market To Limit Orders.
    """
    LIMIT = "limit"
    MARKET = "market"
    MARKET_TO_LIMIT = "market_to_limit"


class MarketDataEntry(Enum):
    """Identifies market data entries for an instrument.
    BIDS: Best buy offer in the Market Book.
    OFFERS: Best sell offer in the Market Book.
    LAST: Last price traded in the Market Book.
    OPENING_PRICE: Opening price in the Market Book.
    CLOSING_PRICE: Closing price in the Market Book.
    SETTLEMENT_PRICE: Settlement price (only for futures)
    HIGH_PRICE: Highest price traded.
    LOW_PRICE: Lowest price traded.
    TRADE_VOLUME: Traded volume in contracts/nominal.
    OPEN_INTEREST: Open interest in contracts (only for futures)
    INDEX_VALUE: Calculated Index Value (only for Index)
    TRADE_EFFECTIVE_VOLUME: Effective traded volume.
    NOMINAL_VOLUME: Nominal traded volume.
    """
    BIDS = "BI"
    OFFERS = "OF"
    LAST = "LA"
    OPENING_PRICE = "OP"
    CLOSING_PRICE = "CL"
    SETTLEMENT_PRICE = "SE"
    HIGH_PRICE = "HI"
    LOW_PRICE = "LO"
    TRADE_VOLUME = "TV"
    OPEN_INTEREST = "OI"
    INDEX_VALUE = "IV"
    TRADE_EFFECTIVE_VOLUME = "EV"
    NOMINAL_VOLUME = "NV"



# -*- coding: utf-8 -*-
"""
    pyRofex.service

    All the library exposed functionality
"""
from inspect import getargspec

from .clients.rest_rfx import RestClient
from .clients.websocket_rfx import WebSocketClient
from .components import globals
from .components.exceptions import ApiException
from .components.enums import Environment
from .components.enums import MarketDataEntry
from .components.enums import TimeInForce
from .components.enums import Market


"""
************* Initialization functions *************
"""


def initialize(user, password, account, environment):
    """ Initialize the specified environment.

     Set the default user, password and account for the environment.

    :param user: the user used for authentication.
    :type user: str
    :param password: the password used for authentication.
    :type password: str
    :param account: user's default account.
    :type account: str
    :param environment: the environment that is gonna be initialized.
    :type environment: Environment (Enum)
    """
    _validate_environment(environment)
    _set_environment_parameters(user, password, account, environment)
    globals.environment_config[environment]["rest_client"] = RestClient(environment)
    globals.environment_config[environment]["ws_client"] = WebSocketClient(environment)
    set_default_environment(environment)


def set_default_environment(environment):
    """Set default environment.

    The environment that is gonna be used as default when it's not specified.

    Example: if we send an order using the send_order function and
    we do not specified and Environment, the order is going to be send to the default one.

    :param environment: the environment that is going to be set as default.
    :type environment: Environment
    """
    _validate_environment(environment)
    globals.default_environment = environment


def _set_environment_parameters(user, password, account, environment):
    """Configure the environment parameters into global configuration.

    Set the user, password and account into globals configuration.
    These variables are going to be used as the default parameters for the environment.

    :param user: the user used for authentication.
    :param password: the password used for authentication.
    :param account: user's default account.
    :param environment: the environment that is going to be configured.
    """
    globals.environment_config[environment]["user"] = user
    globals.environment_config[environment]["password"] = password
    globals.environment_config[environment]["account"] = account


"""
************* REST functions *************
"""


def get_segments(environment=None):
    """Make a request to the API and get a list of valid segments.

    For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

    :param environment: The environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    :return: A list of valid ROFEXs segments returned by the API.
    :rtype: dict of JSON response.
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)

    # Get the client for the environment and start the connection
    response = globals.environment_config[environment]["rest_client"].get_segments()
    return response


def get_all_instruments(environment=None):
    """Make a request to the API and get a list of all available instruments.

    For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    :return: A list of valid instruments returned by the API.
    :rtype: dict of JSON response.
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)

    # Get the client for the environment and make the request
    client = globals.environment_config[environment]["rest_client"]
    return client.get_all_instruments()


def get_detailed_instruments(environment=None):
    """Make a request to the API and get a detailed list of all available instruments.

    For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    :return: A list of valid instruments returned by the API.
    :rtype: dict of JSON response.
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)

    # Get the client for the environment and make the request
    client = globals.environment_config[environment]["rest_client"]
    return client.get_detailed_instruments()


def get_market_data(ticker, entries, depth=1, market=Market.ROFEX, environment=None):
    """Make a request to the API to get the Market Data Entries of the specified instrument.

    For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

    :param ticker: Instrument symbol to send in the request. Example: DODic19
    :type ticker: str
    :param entries: List of entries to send in the request. Example: [MarketDataEntry.BIDS, MarketDataEntry.OFFERS]
    :type entries: List of MarketDataEntry (Enum).
    :param depth: Specify the depth of the book to be request. Default: 1.
    :type depth: int
    :param market: Market ID related to the instrument. Default Market.ROFEX.
    :type market: Market (Enum).
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    :return: Market Data response of the API.
    :rtype: dict of JSON response.
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)
    _validate_market_data_entries(entries)

    # Get the client for the environment and make the request
    client = globals.environment_config[environment]["rest_client"]
    return client.get_market_data(ticker, entries, depth, market)


def get_order_status(client_order_id, proprietary=None, environment=None):
    """Make a request to the API to get the status of the specified order.

    For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

    :param client_order_id: Client Order ID of the order.
    :type client_order_id: str
    :param proprietary: Proprietary of the order. Default None: environment default proprietary is used.
    :type proprietary: str
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    :return: Order status response of the API.
    :rtype: dict of JSON response.
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)

    # Checks the proprietary and sets the default one if None is received.
    if proprietary is None:
        proprietary = globals.environment_config[environment]["proprietary"]

    # Get the client for the environment and make the request
    client = globals.environment_config[environment]["rest_client"]
    return client.get_order_status(client_order_id,
                                   proprietary)


def send_order(ticker, size, order_type, side,
               account=None,
               price=None,
               time_in_force=TimeInForce.DAY,
               market=Market.ROFEX,
               cancel_previous=False,
               environment=None):
    """Make a request to the API that send a new order to the Market.

    For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

    :param ticker: Instrument symbol to send in the request. Example: DODic19.
    :type ticker: str
    :param size: Order size.
    :type size: int
    :param order_type: Order type. Example: OrderType.LIMIT.
    :type order_type: OrderType (Enum).
    :param side: Order side. Example: Side.BUY.
    :type side: Side (Enum).
    :param account: Account to used. Default None: default account is used.
    :type account: str
    :param price: Order price. Default None: when no price is required.
    :type price: int
    :param time_in_force: Order modifier that defines the active time of the order. Default TimeInForce.Day.
    :type time_in_force: TimeInForce (Enum).
    :param market: Market ID related to the instrument. Default Market.ROFEX.
    :type market: Market (Enum).
    :param cancel_previous: True: cancels actives orders that match with the account, side and ticker.
    False: send the order without cancelling previous ones. Useful for replacing old orders. Default: False.
    :type cancel_previous: boolean.
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    :return: Client Order ID and Proprietary of the order returned by the API.
    :rtype: dict of JSON response.
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)
    _validate_account(account, environment)

    # Checks the account and sets the default one if None is received.
    if account is None:
        account = globals.environment_config[environment]["account"]

    # Get the client for the environment and make the request
    client = globals.environment_config[environment]["rest_client"]
    return client.send_order(ticker, size, order_type, side, account,
                             price, time_in_force, market, cancel_previous)


def cancel_order(client_order_id, proprietary=None, environment=None):
    """Make a request to the API and cancel the order specified.

    The market will respond with a client order id, then you should verify the status of the request with this id.

    For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

    :param client_order_id: Client Order ID of the order.
    :type client_order_id: str
    :param proprietary: Proprietary of the order. Default None: environment default proprietary is used.
    :type proprietary: str
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    :return: Client Order ID of cancellation request returned by the API.
    :rtype: dict of JSON response.
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)

    # Checks the proprietary and sets the default one if None is received.
    if proprietary is None:
        proprietary = globals.environment_config[environment]["proprietary"]

    # Get the client for the environment and make the request
    client = globals.environment_config[environment]["rest_client"]
    return client.cancel_order(client_order_id,
                               proprietary)


def get_all_orders_status(account=None, environment=None):
    """Make a request to the API and get the status of all the orders associated with the account.

    For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

    :param account: Account associated with the orders. Default None: default account is used.
    :type account: str
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    :return: List of all orders status associated with the user returned by the API.
    :rtype: dict of JSON response.
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)
    _validate_account(account, environment)

    # Checks the account and sets the default one if None is received.
    if account is None:
        account = globals.environment_config[environment]["account"]

    # Get the client for the environment and make the request
    client = globals.environment_config[environment]["rest_client"]
    return client.get_all_orders_by_account(account)


def get_trade_history(ticker, start_date, end_date, market=Market.ROFEX, environment=None):
    """Makes a request to the API and get trade history for the instrument specified.

    For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

    :param ticker: Instrument symbol to send in the request. Example: DODic19
    :type ticker: str
    :param start_date: Start date for the trades. Format: yyyy-MM-dd
    :type start_date: str
    :param end_date: End date for the trades. Format: yyyy-MM-dd
    :type end_date: str
    :param market: Market ID related to the instrument. Default Market.ROFEX.
    :type market: Market (Enum).
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    :return: List of trades returned by the API.
    :rtype: dict of JSON response.
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)

    # Get the client for the environment and make the request
    client = globals.environment_config[environment]["rest_client"]
    return client.get_trade_history(ticker, start_date, end_date, market)


""" 
************* Websocket functions ************* 
"""


def init_websocket_connection(market_data_handler=None,
                              order_report_handler=None,
                              error_handler=None,
                              exception_handler=None,
                              environment=None):
    """Initialize the Websocket Client with the handlers and then start the connection with Primary Websocket API.

    A new thread is created in order to motorize the connection and check new incoming messages.

    :param market_data_handler: function called when a new Market Data Message is received. Default None.
    :type market_data_handler: callable.
    :param order_report_handler: function called when a new Order Report Message is received. Default None.
    :type order_report_handler: callable.
    :param error_handler: function called when an Error Message is received. Default None.
    :type error_handler: callable.
    :param exception_handler: function called when an Exception occurred in the client. Default None.
    :type exception_handler: callable.
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)

    # Gets the client for the environment
    client = globals.environment_config[environment]["ws_client"]

    # Checks handlers and adds the them into the client.
    if market_data_handler is not None:
        _validate_handler(market_data_handler)
        client.add_market_data_handler(market_data_handler)

    if order_report_handler is not None:
        _validate_handler(order_report_handler)
        client.add_order_report_handler(order_report_handler)

    if error_handler is not None:
        _validate_handler(error_handler)
        client.add_error_handler(error_handler)

    if exception_handler is not None:
        _validate_handler(error_handler)
        client.add_error_handler(exception_handler)

    # Initiates the connection with the Websocket API
    client.connect()


def close_websocket_connection(environment=None):
    """Close the connection with the API.

    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    """

    # Validations
    environment = _validate_environment(environment)

    # Gets the client for the environment
    client = globals.environment_config[environment]["ws_client"]

    # Close Websocket connection with the API
    client.close_connection()


def order_report_subscription(account=None, snapshot=True, handler=None, environment=None):
    """Send an Order Report Subscription Message through the connection.

    :param account: account that will be send in the message.
    Default None: default environment account is used.
    :type account: str.
    :param snapshot: True: old Order Reports won't be received; False: old Order Report will be received. Default True.
    :type snapshot: boolean.
    :param handler: function that is going to be call when a new Order Report Message is received. Default None.
    :type handler: callable.
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)
    _validate_websocket_connection(environment)
    _validate_account(account, environment)

    # Checks the account and sets the default one if None is received.
    if account is None:
        account = globals.environment_config[environment]["account"]

    # Checks the handler, then validates and adds it into the client.
    if handler is not None:
        _validate_handler(handler)
        globals.environment_config[environment]["ws_client"].add_order_report_handler(handler)

    # Get the client for the environment and send the subscription message
    client = globals.environment_config[environment]["ws_client"]
    client.order_report_subscription(account, snapshot)


def market_data_subscription(tickers, entries, market=Market.ROFEX, handler=None, environment=None):
    """Send a Market Data Subscription Message through the connection.

    :param tickers: list of the the instruments to be subscribe.
    :type tickers: List of str.
    :param entries: List of market data entries that want to be received.
    Example: [MarketDataEntry.BIDS, MarketDataEntry.OFFERS]
    :type entries: List of MarketDataEntry (Enum).
    :param market: Market id associated to the tickers. Default Market.ROFEX.
    :type market: Market (Enum).
    :param handler: function that is going to be call when a new Market Data Message is received. Default None.
    :type handler: callable.
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_initialization(environment)
    _validate_websocket_connection(environment)
    _validate_market_data_entries(entries)

    # Checks the handler, then validates and adds it into the client.
    if handler is not None:
        _validate_handler(handler)
        globals.environment_config[environment]["ws_client"].add_market_data_handler(handler)

    # Get the client for the environment and send the subscription message
    client = globals.environment_config[environment]["ws_client"]
    client.market_data_subscription(tickers, entries, market)


def add_websocket_market_data_handler(handler, environment=None):
    """Adds a new Market Data handler to the Websocket Client.

    This handler is going to be call when a new Market Data Message is received.

    :param handler: function that is going to be call when a new Market Data Message is received.
    :type handler: callable.
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_handler(handler)

    # Get the client for the environment and adds the handler
    client = globals.environment_config[environment]["ws_client"]
    client.add_market_data_handler(handler)


def add_websocket_order_report_handler(handler, environment=None):
    """Adds a new Order Report handler to the Websocket Client.

    This handler is going to be call when a new Order Report Message is received.

    :param handler: function that is going to be call when a new Order Report Message is received.
    :type handler: callable.
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_handler(handler)

    # Get the client for the environment and adds the handler
    client = globals.environment_config[environment]["ws_client"]
    client.add_order_report_handler(handler)


def add_websocket_error_handler(handler, environment=None):
    """Adds a new Error handler to the Websocket Client.

    This handler is going to be call when a new Error Message is received.

    :param handler: function that is going to be call when a new Error Message is received.
    :type handler: callable.
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_handler(handler)

    # Get the client for the environment and adds the handler
    client = globals.environment_config[environment]["ws_client"]
    client.add_error_handler(handler)


def set_websocket_exception_handler(handler, environment=None):
    """Set the Exception handler to the Websocket Client.

    This handler is going to be called when an Exception occurred in the client.

    :param handler: function called when Exception is raised.
    :type handler: callable.
    :param environment: Environment used. Default None: the default environment is used.
    :type environment: Environment (Enum).
    """

    # Validations
    environment = _validate_environment(environment)
    _validate_handler(handler)

    # Get the client for the environment and adds the handler
    client = globals.environment_config[environment]["ws_client"]
    client.set_exception_handler(handler)


"""
************* Validations functions *************
"""


def _validate_environment(environment):
    """Check if the environment sent is an instance of the Environment enum.

    If None environment is send then check if default environment is set.

    If none environment is set or the environment is not an instance of Environment raise an ApiException.

    :param environment: Environment to be validated.
    :type environment: Environment (Enum).
    :return: the environment to be used.
    :rtype: Environment (Enum).
    """
    if environment is None:
        if globals.default_environment is None:
            raise ApiException("Environment not specify.")
        else:
            environment = globals.default_environment

    if not isinstance(environment, Environment):
        raise ApiException("Invalid Environment.")

    return environment


def _validate_initialization(environment):
    """Validate if the environment is initialized. If not it raised an ApiException.

    :param environment: Environment used.
    :type environment: Environment (Enum).
    """
    if not globals.environment_config[environment]["initialized"]:
        raise ApiException("The Environment is not initialized.")


def _validate_market_data_entries(entries):
    """Validate if the list of entries are instance of the MarketDataEntry enum.

    :param entries: list of entries to be validated.
    :type entries: list of MarketDataEntry (Enum).
     """
    for entry in entries:
        if not isinstance(entry, MarketDataEntry):
            raise ApiException("Invalid Market Data Entry: " + str(entry))


def _validate_websocket_connection(environment):
    """Checks if the websocket connection was established.

    If the client is not connected then tries to initialize the connection.

    :param environment: Environment used.
    :type environment: Environment (Enum).
    """
    if not globals.environment_config[environment]["ws_client"].is_connected():
        globals.environment_config[environment]["ws_client"].connect()


def _validate_account(account, environment):
    """Checks if the account if None account is send, then the environment account
    must be set, if not raised an ApiException.

    :param account: account to be validated.
    :type account: str.
    :param environment: Environment used.
    :type environment: Environment (Enum).
    """
    if account is None and globals.environment_config[environment]["account"] is None:
        raise ApiException("Account not specified.")


def _validate_handler(handler):
    """ Checks if the handler is callable and that can received one argument, if not raised an ApiException.

    :param handler: handler to be validated.
    :type handler: callable.
    """

    # Checks if it is callable
    if not callable(handler):
        raise ApiException("Handler '{handler}' is not callable.".format(handler=handler))

    # Checks if function can receive an argument
    fun_arg_spec = getargspec(handler)
    if not fun_arg_spec.args and not fun_arg_spec.varargs:
        print("Handler '{handler}' can't receive an argument.".format(handler=handler))



# -*- coding: utf-8 -*-
"""
    pyRofex: Python connector for ROFEX Rest and Websocket APIs.

    This modules expose functions and enumerations of the library.
"""
from .service import initialize
from .service import set_default_environment

from .service import init_websocket_connection
from .service import close_websocket_connection
from .service import market_data_subscription
from .service import order_report_subscription
from .service import add_websocket_market_data_handler
from .service import add_websocket_order_report_handler
from .service import add_websocket_error_handler
from .service import set_websocket_exception_handler

from .service import get_segments
from .service import get_all_instruments
from .service import get_detailed_instruments
from .service import get_market_data
from .service import get_trade_history
from .service import send_order
from .service import cancel_order
from .service import get_order_status
from .service import get_all_orders_status

from .components.enums import Environment
from .components.enums import MarketDataEntry
from .components.enums import Market
from .components.enums import OrderType
from .components.enums import Side
from .components.enums import TimeInForce

__version__ = "0.1.0.rc1"



# -*- coding: utf-8 -*-
"""
    pyRofex.rest_client

    Defines a Rest Client that implements ROFEX Rest API.
"""
import requests
import simplejson

from ..components import urls
from ..components import globals
from ..components.exceptions import ApiException


class RestClient:
    """ Rest Client that implements call to ROFEX REST API.

    For more information about the API go to: http://api.primary.com.ar/docs/Primary-API.pdf
    """

    def __init__(self, environment):
        """Initialization of the Client.

        :param environment: the environment that will be associated with the client.
        :type environment: Environment (Enum)
         """

        # Environment associated with Client
        self.environment = globals.environment_config[environment]

        # Get the authentication Token.
        self.update_token()

    def get_trade_history(self, ticker, start_date, end_date, market):
        """Makes a request to the API and get trade history for the instrument.

        For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

        :param ticker: Ticker of the instrument to send in the request. Example: DODic19
        :type ticker: str
        :param start_date: Start date for the trades. Format: yyyy-MM-dd
        :type start_date: str
        :param end_date: End date for the trades. Format: yyyy-MM-dd
        :type end_date: str
        :param market: Market ID related to the instrument.
        :type market: Market (Enum).
        :return: List of trades returned by the API.
        :rtype: dict of JSON response.
        """
        return self.api_request(urls.historic_trades.format(m=market.value,
                                                            s=ticker,
                                                            df=start_date,
                                                            dt=end_date))

    def get_segments(self):
        """Make a request to the API and get a list of valid segments.

        For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

        :return: A list of valid ROFEXs segments returned by the API.
        :rtype: dict of JSON response.
        """
        return self.api_request(urls.segments)

    def get_all_instruments(self):
        """Make a request to the API and get a list of all available instruments.

        For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

        :return: A list of valid instruments returned by the API.
        :rtype: dict of JSON response.
        """
        return self.api_request(urls.instruments)

    def get_detailed_instruments(self):
        """Make a request to the API and get a list of all available instruments.

        For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

        :return: A list of valid instruments returned by the API.
        :rtype: dict of JSON response.
        """
        return self.api_request(urls.detailed_instruments)

    def get_market_data(self, ticker, entries, depth, market):
        """Make a request to the API to get the Market Data Entries of the specified instrument.

        For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

        :param ticker: Instrument symbol to send in the request. Example: DODic19
        :type ticker: str
        :param entries: List of entries to send in the request. Example: [MarketDataEntry.BIDS, MarketDataEntry.OFFERS]
        :type entries: List of MarketDataEntry (Enum).
        :param depth: Specify the depth of the book to be request. Default: 1.
        :type depth: int
        :param market: Market ID related to the instrument.
        :type market: Market (Enum).
        :return: Market Data response of the API.
        :rtype: dict of JSON response.
        """

        # Creates a comma separated string with the entries in the list.
        entry_string = ",".join([entry.value for entry in entries])
        return self.api_request(urls.market_data.format(m=market.value,
                                                        s=ticker,
                                                        e=entry_string,
                                                        d=depth))

    def get_order_status(self, client_order_id, proprietary):
        """Make a request to the API to get the status of the specified order.

        For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

        :param client_order_id: Client Order ID of the order.
        :type client_order_id: str
        :param proprietary: Proprietary of the order.
        :type proprietary: str
        :return: Order status response of the API.
        :rtype: dict of JSON response.
        """
        return self.api_request(urls.order_status.format(c=client_order_id,
                                                         p=proprietary))

    def get_all_orders_by_account(self, account):
        """Make a request to the API and get the status of all the orders associated with the account.

        For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

        :param account: Account associated with the orders.
        :type account: str
        :return: List of all orders status associated with the user returned by the API.
        :rtype: dict of JSON response.
        """
        return self.api_request(urls.all_orders_status.format(a=account))

    def send_order(self, ticker, size, order_type, side, account, price, time_in_force, market, cancel_previous):
        """Make a request to the API that send a new order to the Market.

        For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

        :param ticker: Instrument symbol to send in the request. Example: DODic19.
        :type ticker: str
        :param size: Order size.
        :type size: int
        :param order_type: Order type. Example: OrderType.LIMIT.
        :type order_type: OrderType (Enum).
        :param side: Order side. Example: Side.BUY.
        :type side: Side (Enum).
        :param account: Account to used.
        :type account: str
        :param price: Order price.
        :type price: int
        :param time_in_force: Order modifier that defines the active time of the order.
        :type time_in_force: TimeInForce (Enum).
        :param market: Market ID related to the instrument.
        :type market: Market (Enum).
        :param cancel_previous: True: cancels actives orders that match with the account, side and ticker.
        False: send the order without cancelling previous ones. Useful for replacing old orders. Default: False.
        :type cancel_previous: boolean.
        :return: Client Order ID and Proprietary of the order returned by the API.
        :rtype: dict of JSON response.
        """
        return self.api_request(urls.new_order[order_type].format(market=market.value,
                                                                  ticker=ticker,
                                                                  size=size,
                                                                  type=order_type.value,
                                                                  side=side.value,
                                                                  time_force=time_in_force.value,
                                                                  account=account,
                                                                  price=price,
                                                                  cancel_previous=cancel_previous))

    def cancel_order(self, client_order_id, proprietary):
        """Make a request to the API and cancel the order specified.

        The market will respond with a client order id, then you should verify the status of the request with this id.

        For more detailed information go to: http://api.primary.com.ar/docs/Primary-API.pdf

        :param client_order_id: Client Order ID of the order.
        :type client_order_id: str
        :param proprietary: Proprietary of the order.
        :type proprietary: str
        :return: Client Order ID of cancellation request returned by the API.
        :rtype: dict of JSON response.
        """
        return self.api_request(urls.cancel_order.format(id=client_order_id,
                                                         p=proprietary))

    def api_request(self, path, retry=True):
        """ Make a GET request to the API.

        :param path: path to the API resource.
        :type path: str
        :param retry: True: update the token and resend the request if the response code is 401.
        False: raise an exception if the response code is 401.
        :type proprietary: str
        :return: response of the API.
        :rtype: dict of JSON response.
        """
        headers = {'X-Auth-Token': self.environment["token"]}
        response = requests.get(self._url(path),
                                headers=headers,
                                verify=self.environment["ssl"])

        # Checks if the response code is 401 (Unauthorized)
        if response.status_code == 401:
            if retry:
                self.update_token()
                self.api_request(path, False)
            else:
                raise ApiException("Authentication Fails.")

        return simplejson.loads(response.content)

    def update_token(self):
        """ Authenticate using the environment user and password.

        Then save the token in the environment parameters and set the initialized parameter to True.
        """
        headers = {'X-Username': self.environment["user"],
                   'X-Password': self.environment["password"]}
        response = requests.post(self._url(urls.auth),
                                 headers=headers,
                                 verify=self.environment["ssl"])

        if not response.ok:
            raise ApiException("Authentication fails. Incorrect User or Password")

        self.environment["token"] = response.headers['X-Auth-Token']
        self.environment["initialized"] = True

    def _url(self, path):
        """ Helper function that concatenate the path to the environment url.

        :param path: path to the API resource.
        :type path: str
        :return: URL to be call.
        :rtype: str
        """
        return self.environment["url"] + path



# -*- coding: utf-8 -*-
"""
    pyRofex.components.exceptions

    Defines all library exceptions
"""


class ApiException(Exception):
    """
    Represent a controlled exception raised by the library.
    """
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg




# -*- coding: utf-8 -*-
"""
    pyRofex.components.messages

    Defines APIs messages templates
"""

# Template for a Market Data Subscription message
MARKET_DATA_SUBSCRIPTION = '{{"type":"smd","level":1, "entries":[{entries}],"products":[{symbols}]}}'
# Template for an Order Subscription message
ORDER_SUBSCRIPTION = '{{"type":"os","account":{{"id":"{a}"}},"snapshotOnlyActive":{snapshot}}}'
# Template to specify an instrument in a market data subscription message
INSTRUMENT = '{{"symbol":"{ticker}","marketId":"{market}"}}'
# Template to insert a Double Quote
DOUBLE_QUOTES = '"{item}"'



# -*- coding: utf-8 -*-
"""
    pyRofex.urls

    Defines all API Paths
"""
from .enums import OrderType

auth = "auth/getToken"
segments = "rest/segment/all"
instruments = "rest/instruments/all"
detailed_instruments = "rest/instruments/details"
market_data = "rest/marketdata/get?marketId={m}&symbol={s}&entries={e}&depth={d}"
historic_trades = "rest/data/getTrades?marketId={m}&symbol={s}&dateFrom={df}&dateTo={dt}"
order_status = "rest/order/id?clOrdId={c}&proprietary={p}"
new_order = {
    OrderType.MARKET: "rest/order/newSingleOrder?marketId={market}&symbol={ticker}"
                      "&orderQty={size}&ordType={type}&side={side}&timeInForce={time_force}"
                      "&account={account}&cancelPrevious={cancel_previous}",
    OrderType.LIMIT: "rest/order/newSingleOrder?marketId={market}&symbol={ticker}"
                     "&price={price}&orderQty={size}&ordType={type}&side={side}"
                     "&timeInForce={time_force}&account={account}&cancelPrevious={cancel_previous}"
}
cancel_order = "rest/order/cancelById?clOrdId={id}&proprietary={p}"
all_orders_status = "rest/order/all?accountId={a}"



# -*- coding: utf-8 -*-
"""
    pyRofex.components.globals

    Defines library global variables
"""
from .enums import Environment

# Default environment used if None environment is specified.
default_environment = None

# Environment specific configuration.
environment_config = {
    Environment.REMARKET: {
        "url": "http://api.remarkets.primary.com.ar/",
        "ws": "ws://api.remarkets.primary.com.ar/",
        "ssl": False,
        "rest_client": None,
        "ws_client": None,
        "token": None,
        "user": None,
        "password": None,
        "account": None,
        "initialized": False,
        "proprietary": "PBCP"
    },
    Environment.LIVE: {
        "url": "https://api.primary.com.ar/",
        "ws": "wss://api.primary.com.ar/",
        "ssl": True,
        "rest_client": None,
        "ws_client": None,
        "user": None,
        "password": None,
        "account": None,
        "initialized": False,
        "proprietary": "api"
    }
}



import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyRofex",
    version="0.1.0.rc1",
    author="Franco Zanuso",
    author_email="francozanuso89@gmail.com",
    description="Python connector for ROFEX's Rest and Websocket APIs.",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/gruporofex/pyRofex",
    packages=setuptools.find_packages(where='src'),
    package_dir={'': 'src'},
    install_requires=[
        'requests>=2.20.0',
        'simplejson>=3.10.0',
        'enum34>=1.1.6',
        'websocket-client>=0.54.0',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Topic :: Office/Business :: Financial :: Investment",
        "Topic :: Software Development"
    ],
)


# -*- coding: utf-8 -*-
"""
    pyRofex.websocket_client

    Defines a Websocket Client that connect to ROFEX Websocket API.
"""
import threading
import time

import websocket
import simplejson

from ..components import globals
from ..components import messages
from ..components.exceptions import ApiException


class WebSocketClient():
    """ Websocket Client that connect to Primary Websocket API.

    This client used a websocket implementation of the library websocket_client.

    - For more references of websocket_client library go to: https://pypi.org/project/websocket_client
    - For more information about the API go to: http://api.primary.com.ar/docs/Primary-API.pdf

    """
    def __init__(self, environment):
        """ Initialization of the client.

        Create and initialize instance variables for the client.

        :param environment: the environment that will be associated with the client.
        :type environment: Environment (Enum)
        """

        # Environment associated with Client
        self.environment = globals.environment_config[environment]

        # Handlers for incoming messages
        self.market_data_handlers = []
        self.order_report_handlers = []
        self.error_handlers = []
        self.exception_handler = None

        # Connection related variables
        self.ws_connection = None
        self.ws_thread = None
        self.connected = False

    def add_market_data_handler(self, handler):
        """ Adds a new Market Data handler to the handlers list.

        :param handler: function that is going to be call when a new Market Data Message is received.
        :type handler: callable.
        """
        self.market_data_handlers.append(handler)

    def add_order_report_handler(self, handler):
        """ Adds a new Order Report handler to the handlers list.

        :param handler: function that is going to be call when a new Order Report Message is received.
        :type handler: callable.
        """
        self.order_report_handlers.append(handler)

    def add_error_handler(self, handler):
        """ Adds a new Error handler to the handlers list.

        :param handler: function that is going to be call when a new Error Message is received.
        :type handler: callable.
        """
        self.error_handlers.append(handler)

    def set_exception_handler(self, handler):
        """ Sets the Exception Handler.

        :param handler: function called when Exception is raised.
        :type handler: callable.
        """
        self.exception_handler = handler

    def connect(self):
        """ Start a new websocket connection with ROFEX API.

        Create an instance WebSocketApp using the environment
        It will create a new thread that is going to be listening new incoming messages.
        """
        headers = {'X-Auth-Token:{token}'.format(token=self.environment["token"])}
        self.ws_connection = websocket.WebSocketApp(self.environment["ws"],
                                                    on_message=self.on_message,
                                                    on_error=self.on_error,
                                                    on_close=self.on_close,
                                                    on_open=self.on_open,
                                                    header=headers)

        # Create a thread and target it to the run_forever function, then start it.
        self.ws_thread = threading.Thread(target=self.ws_connection.run_forever,
                                          kwargs={"ping_interval": 270})
        self.ws_thread.start()

        # Wait 5 sec to establish the connection
        conn_timeout = 5
        time.sleep(1)
        while not self.ws_connection.sock.connected and conn_timeout:
            time.sleep(1)
            conn_timeout -= 1
        if not self.ws_connection.sock.connected:
            self.on_exception(ApiException("Connection could not be established."))

    def on_message(self, message):
        """ Called when a new message is received through the connection.

        :param message: message received.
        :type message: str
        """
        try:
            # Transform the JSON string message to a dict.
            msg = simplejson.loads(message)

            # Checks if it is an error message
            if 'status' in msg and msg['status'] == 'ERROR':
                for handler in self.error_handlers:
                    handler(msg)
            elif 'type' in msg:
                # extract the message type.
                msg_type = msg['type'].upper()

                # Checks message type and call the correct handlers
                if msg_type == 'MD':
                    for handler in self.market_data_handlers:
                        handler(msg)
                elif msg_type == 'OR':
                    for handler in self.order_report_handlers:
                        handler(msg)
                else:
                    msg_type_not_supported = "Websocket: Message Type not Supported. Message: {msg}"
                    for handler in self.error_handlers:
                        handler(msg_type_not_supported.format(msg=msg))
            else:
                msg_not_supported = "Websocket: Message Supported. Message: {msg}"
                for handler in self.error_handlers:
                    handler(msg_not_supported.format(msg=msg))

        except Exception as e:
            self.on_exception(e)

    def on_error(self, exception):
        """ Called when an error occurred within the connection.

        :param exception: exception raised.
        :type exception: exception object
        """
        self.ws_connection.close()
        self.on_exception(exception)

    def on_exception(self, exception):
        """Called when an exception occurred within the client.

        :param exception: exception raised.
        :type exception: exception object
        """
        self.exception_handler(exception)

    def on_close(self):
        """ Called when the connection was closed.
        """
        self.connected = False

    def on_open(self):
        """ Called when the connection was opened.
        """
        self.connected = True

    def close_connection(self):
        """ Close the connection.
        """
        self.ws_connection.close()

    def market_data_subscription(self, tickers, entries, market):
        """ Creates and sends new Market Data Subscription Message through the connection.

        :param tickers: List of the tickers to subscribe.
        :type tickers: list of str
        :param entries: List of market data entries that want to be received.
        Example: [MarketDataEntry.BIDS, MarketDataEntry.OFFERS]
        :type entries: List of MarketDataEntry (Enum).
        :param market: Market id associated to the tickers.
        :type market: Market (Enum).
        """

        # Iterates through the tickers list and creates a new list of Instrument String using the INSTRUMENT Template.
        # Then create a comma separated string with the instruments in the list.
        instruments = [messages.INSTRUMENT.format(ticker=ticker, market=market.value) for ticker in tickers]
        instruments_string = ",".join(instruments)

        # Iterates through the entries list and creates a new list with the entry values.
        # Then creates a comma separated string with the entries in the list. Sample Output: '"BI","OF"'
        entries = [messages.DOUBLE_QUOTES.format(item=entry.value) for entry in entries]
        entries_string = ",".join(entries)

        # Creates a Market Data Subscription Message using the Template.
        message = messages.MARKET_DATA_SUBSCRIPTION.format(entries=entries_string,
                                                           symbols=instruments_string)

        # Send the message through the connection.
        self.ws_connection.send(message)

    def order_report_subscription(self, account, snapshot):
        """ Creates and sends new Order Report Subscription Message through the connection.

        :param account: account that will be send in the message.
        :type account: str.
        :param snapshot: True: old Order Reports won't be received; False: old Order Report will be received.
        :type snapshot: boolean.
        """

        # Create an Order Subscription message using the Template and the parameters.
        message = messages.ORDER_SUBSCRIPTION.format(a=account, snapshot=snapshot.__str__().lower())

        # Send the message through the connection.
        self.ws_connection.send(message)

    def is_connected(self):
        """ Checks if the client is connected to the API.

        :return: True: if it is connected. False: if it is not connected.
        :rtype: boolean.
        """
        return self.connected
